#!/usr/bin/env python

import pylikwid


pylikwid.inittopology()
topodict = pylikwid.getcputopology()
freqlist = pylikwid.getavailfreqs(0)
govlist = pylikwid.getavailgovs(0)
print("Available frequencies for CPU core 0:\n{}\n".format(freqlist))
minfreq = float(pylikwid.getcpuclockmin(0))
maxfreq = float(pylikwid.getcpuclockmax(0))
print("Available CPU governors for CPU core 0:\n{}\n".format(govlist))



for idx in topodict["threadPool"]:
    cpu = topodict["threadPool"][idx]["apicId"]
    print("CPU {} : {} Hz (min: {}, max: {}, gov: {})".format(cpu, pylikwid.getcpuclockcurrent(cpu),
                                                          pylikwid.getcpuclockmin(cpu),
                                                          pylikwid.getcpuclockmax(cpu),
                                                          pylikwid.getgovernor(cpu)))
minunc = int(pylikwid.getuncoreclockmin(0)/1E6)
maxunc = int(pylikwid.getuncoreclockmax(0)/1E6)
print("\nUncore frequencies:")
for socket in range(topodict["numSockets"]):
    print("Socket {} : min: {} MHz, max: {} MHz".format(socket, pylikwid.getuncoreclockmin(socket),
                                                                pylikwid.getuncoreclockmax(socket)))

print("\nSet frequency of CPU 1 to minimum {} MHz:".format(int(float(minfreq)/1E6)))
pylikwid.setcpuclockmin(1, int(float(minfreq)/1E3))
pylikwid.setcpuclockmax(1, int(float(minfreq)/1E3))
print("CPU {} : {} Hz (min: {}, max: {}, gov: {})".format(1, pylikwid.getcpuclockcurrent(1),
                                                          pylikwid.getcpuclockmin(1),
                                                          pylikwid.getcpuclockmax(1),
                                                          pylikwid.getgovernor(1)))

print("\nReset frequency of CPU 1:")
pylikwid.setcpuclockmin(1, int(float(minfreq)/1E3))
pylikwid.setcpuclockmax(1, int(float(maxfreq)/1E3))
print("CPU {} : {} Hz (min: {}, max: {}, gov: {})".format(1, pylikwid.getcpuclockcurrent(1),
                                                          pylikwid.getcpuclockmin(1),
                                                          pylikwid.getcpuclockmax(1),
                                                          pylikwid.getgovernor(1)))

gov = pylikwid.getgovernor(1)
oldgov = gov
for g in govlist.split(" "):
    if g != gov:
        gov = g
        break

print("\nSet governor of CPU 1 to {}:".format(gov))
pylikwid.setgovernor(1, gov)
print("CPU {} : {} Hz (min: {}, max: {}, gov: {})".format(1, pylikwid.getcpuclockcurrent(1),
                                                          pylikwid.getcpuclockmin(1),
                                                          pylikwid.getcpuclockmax(1),
                                                          pylikwid.getgovernor(1)))
print("\nReset governor of CPU 1 to {}:".format(oldgov))
pylikwid.setgovernor(1, oldgov)
print("CPU {} : {} Hz (min: {}, max: {}, gov: {})".format(1, pylikwid.getcpuclockcurrent(1),
                                                          pylikwid.getcpuclockmin(1),
                                                          pylikwid.getcpuclockmax(1),
                                                          pylikwid.getgovernor(1)))

print("\nSet Uncore frequency of socket 0 to minimum {}:".format(minunc))
pylikwid.setuncoreclockmin(0, minunc)
pylikwid.setuncoreclockmax(0, minunc)
print("Socket {} : min: {} MHz, max: {} MHz".format(0, pylikwid.getuncoreclockmin(0),
                                                       pylikwid.getuncoreclockmax(0)))

print("\nReset Uncore frequency of socket 0:")
pylikwid.setuncoreclockmin(0, minunc)
pylikwid.setuncoreclockmax(0, maxunc)
print("Socket {} : min: {} MHz, max: {} MHz".format(0, pylikwid.getuncoreclockmin(0),
                                                       pylikwid.getuncoreclockmax(0)))


pylikwid.finalizetopology()



#!/usr/bin/env python

import pylikwid

def do_work():
    l = []
    for i in range(1000000):
        l += [i]
    print("Running on CPU {}".format(pylikwid.getprocessorid()))


pylikwid.inittopology()
cputopo = pylikwid.getcputopology()
for t in cputopo["threadPool"]:
    pylikwid.pinprocess(cputopo["threadPool"][t]["apicId"])
    do_work()

pylikwid.finalizetopology()



#!/usr/bin/env python

import os, sys, re, time, math

try:
    import pylikwid
except ImportError:
    print("Cannot load LIKWID python module")
    sys.exit(1)

groups = ["L3"]
sleeptime = 1
if len(groups) == 0:
    print("You have to add a performance group name to 'groups' list")
    sys.exit(1)
if sleeptime <= 0:
    print("Sleep time must be greater than zero")
    sys.exit(1)


gids = {}
cpus = []

ret = pylikwid.hpminit()
if not ret:
    print('Failed to initialize access layer for LIKWID')
    sys.exit(1)


ret = pylikwid.inittopology()
if not ret:
    print('Failed to initialize LIKWID topology module')
    sys.exit(1)


topo = pylikwid.getcputopology()
for t in topo["threadPool"].keys():
    cpus.append(topo["threadPool"][t]["apicId"])


ret = pylikwid.init(cpus)
if ret != 0:
    print('Failed to initialize LIKWID perfmon module')
    sys.exit(1)
os.environ["LIKWID_FORCE"] = "1"

if pylikwid.setverbosity(0) != 0:
    print('Failed to set verbosity')
    sys.exit(1)


run = True
try:
    while run:
        for grp in groups:
            if not gids.has_key(grp):
                gid = pylikwid.addeventset(grp)
                if gid < 0:
                    print('Failed to add group {} to LIKWID perfmon module'.format(grp))
                    groups.remove(grp)
                    continue
                gids[grp] = gid
            gid = gids[grp]
            timestamp = time.time()
            pylikwid.setup(gid)
            pylikwid.start()
            time.sleep(sleeptime)
            pylikwid.stop()
            for i in range(len(cpus)):
                for m in range(pylikwid.getnumberofmetrics(gid)):
                    metricname = pylikwid.getnameofmetric(gid, m)
                    v = float(pylikwid.getlastmetric(gid, m, i))
                    if math.isnan(v) or str(v) == "nan":
                        print("Metric {} on CPU {} failed".format(metricname, cpus[i]))
                        run = False
                    elif i == 0:
                        print("{},cpu={} {} {}".format(metricname, cpus[i], v, timestamp))
except KeyboardInterrupt:
    pass

pylikwid.finalize()



#!/usr/bin/env python

import pylikwid

def int_join(l, sep):
    if l and len(l) > 0:
        return sep.join([str(x) for x in l])
    return "\"\""

aff = pylikwid.initaffinity()

for k in aff:
    if isinstance(aff[k], int):
        print("{}: {}".format(k, aff[k]))
print("")

for d in aff["domains"]:
    print("Domain {}:".format(aff["domains"][d]["tag"]))
    print("\t"+int_join(aff["domains"][d]["processorList"], " "))
    print("")

for sel in ["S0:0-3", "N:3-1", "1,2,3", "E:N:2:1:2"]:
    l = pylikwid.cpustr_to_cpulist(sel)
    print("CPU string {} results in {}".format(sel, int_join(l, ",")))

pylikwid.finalizeaffinity()



#!/usr/bin/env python

import pylikwid


pylikwid.markerinit()
pylikwid.markerthreadinit()
liste = []
pylikwid.markerstartregion("liste")
for i in range(0,1000000):
    liste.append(i)

pylikwid.markerstopregion("liste")
nr_events, elist, time, count = pylikwid.markergetregion("liste")
for i, e in enumerate(elist):
    print(i, e)
pylikwid.markerclose()



#!/usr/bin/env python

import sys
import pylikwid

liste = []
cpus = [0,1]
eventset = "INSTR_RETIRED_ANY:FIXC0"

err = pylikwid.init(cpus)
if err > 0:
    print("Cannot initialize LIKWID")
    sys.exit(1)
group = pylikwid.addeventset(eventset)
if group >= 0:
    print("Eventset {} added with ID {}".format(eventset, group,))
else:
    print("Failed to add eventset {}".format(eventset))
    sys.exit(1)
err = pylikwid.setup(group)
if err < 0:
    print("Setup of group {} failed".format(group))
    sys.exit(1)
err = pylikwid.start()
if err < 0:
    print("Start of group {} failed".format(group))
    sys.exit(1)
for i in range(0,1000000):
    liste.append(i)
err = pylikwid.stop()
if err < 0:
    print("Stop of group {} failed".format(group))
    sys.exit(1)
for thread in range(0,len(cpus)):
    print("Result CPU {} : {}".format(cpus[thread], pylikwid.getresult(group,0,thread)))
pylikwid.finalize()



import os, os.path, glob, re, sys
from distutils.core import setup, Extension


# Utility function to read the README.md file.
# Used for the long_description.  It"s nice, because now 1) we have a top level
# README.md file and 2) it"s easier to type in the README.md file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


ver_regex = re.compile(b"so.(\d+)[.]*(\d*)")


def generic_iglob(pattern):
    result = None
    if sys.version_info.major == 3 and sys.version_info.minor > 4:
        result = glob.iglob(pattern, recursive=True)
    else:
        result = glob.iglob(pattern)
    return result


def find_file(path_iterator):
    path = None
    path_to_explore = None
    is_searching = True
    if path_iterator:
        while is_searching:
            try:
                path_to_explore = next(path_iterator)
                if os.path.exists(path_to_explore):
                    path = path_to_explore
                is_searching = False
            except StopIteration:
                is_searching = False
    return path


def get_hierarchy():
    print("Searching for LIKWID installation")
    include_path = None
    library_path = None
    library = None
    prefix_path = os.getenv('LIKWID_PREFIX', None)
    library_pattern = '/lib*/liblikwid.so*'
    if prefix_path is not None:
        iterator = generic_iglob(prefix_path + library_pattern)
        library = find_file(iterator)
        if library is not None:
            library_path = os.path.dirname(library)
    if library is None:
        paths_iterator = iter(os.environ["PATH"].split(":"))
        is_searching = True
        while is_searching:
            try:
                path = next(paths_iterator)
                prefix_path = os.path.abspath(path + '../')
                iterator = generic_iglob(prefix_path + library_pattern)
                library = find_file(iterator)
                if library is not None:
                    library_path = os.path.dirname(library)
                    is_searching = False
            except StopIteration:
                is_searching = False
        if library is None:
            prefix_path = '/usr/local'
            iterator = generic_iglob(prefix_path + library_pattern)
            library = find_file(iterator)
            if library is not None:
                library_path = os.path.dirname(library)

    include_path = os.path.join(prefix_path, 'include')
    if not os.path.exists(os.path.join(include_path, 'likwid.h')):
        iterator = generic_iglob(prefix_path + '**/likwid.h')
        include_path = find_file(iterator)
    if prefix_path is None or not os.path.exists(prefix_path):
        raise Exception('Error the likwid prefix directory was not found')
    if library is None or not os.path.exists(library):
        raise Exception('Error the likwid library was not found')
    if library_path is None or not os.path.exists(library_path):
        raise Exception('Error the likwid library directory was not found')
    if include_path is None or not os.path.exists(include_path):
        raise Exception('Error the likwid include directory was not found')
    print("Using LIKWID prefix directory at {!s}".format(prefix_path))
    print("Using LIKWID include directory at {!s}".format(include_path))
    print("Using LIKWID library directory at {!s}".format(library_path))
    print("Using LIKWID library named {!s}".format(library))
    m = re.match("lib(.*)\.so", os.path.basename(library))
    if m:
        library = m.group(1)
    return prefix_path, library_path, library, include_path

try:
    LIKWID_PREFIX, LIKWID_LIBPATH, LIKWID_LIB, LIKWID_INCPATH = get_hierarchy()
    print(LIKWID_PREFIX, LIKWID_LIBPATH, LIKWID_LIB, LIKWID_INCPATH)
except Exception as e:
    print(e)
    sys.exit(1)


pylikwid = Extension("pylikwid",
                     include_dirs=[LIKWID_INCPATH],
                     libraries=[LIKWID_LIB],
                     library_dirs=[LIKWID_LIBPATH],
                     sources=["pylikwid.c"])

setup(
    name="pylikwid",
    version="0.4.0",
    author="Thomas Roehl",
    author_email="thomas.roehl@googlemail.com",
    description="A Python module to access the function of the LIKWID library",
    long_description=read("README.rst"),
    license="GPLv2",
    keywords="hpc performance benchmark analysis",
    url="https://github.com/RRZE-HPC/pylikwid",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
    ],
    package_data={
        "pylikwid": ["pylikwid.c", "README.rst", "LICENSE"],
        "tests": ["tests/*.py"]
    },
    ext_modules=[pylikwid]
)



#!/usr/bin/env python

import pylikwid

pylikwid.inittopology()
infodict = pylikwid.getcpuinfo()
for k in infodict:
    print("{}: {}".format(k, infodict[k]))
topodict = pylikwid.getcputopology()
for k in topodict:
    if not isinstance(topodict[k], dict):
        print("{}: {}".format(k, topodict[k]))
print()
print("CPU topology:")
print("ID\tCore\tThread\tPackage")
for t in topodict["threadPool"]:
    print("{}\t{}\t{}\t{}".format(topodict["threadPool"][t]["apicId"],
                                  topodict["threadPool"][t]["coreId"],
                                  topodict["threadPool"][t]["threadId"],
                                  topodict["threadPool"][t]["packageId"]))

pylikwid.finalizetopology()
